<?php
// Own namespace
namespace Org\Mxchange\CoreFramework\Tests\Filter\Configuration\Classes;

// Import framework stuff
use Org\Mxchange\CoreFramework\Bootstrap\FrameworkBootstrap;
use Org\Mxchange\CoreFramework\Filter\Filterable;
use Org\Mxchange\CoreFramework\Request\Requestable;
use Org\Mxchange\CoreFramework\Response\Responseable;
use Org\Mxchange\CoreFramework\Tests\Filter\BaseTestsFilter;

// Import SPL stuff
use \InvalidArgumentException;

/**
 * A LoadableClasses filter for tests
 *
 * @author		Roland Haeder <webmaster@ship-simu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2007, 2008 Roland Haeder, 2009 - 2020 Core Developer Team
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.ship-simu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
class TestConfigurationLoadableClassesFilter extends BaseTestsFilter implements Filterable {
	/**
	 * Protected constructor
	 *
	 * @return	void
	 */
	protected function __construct () {
		// Call parent constructor
		parent::__construct(__CLASS__);
	}

	/**
	 * Creates an instance of this filter class
	 *
	 * @return	$filterInstance		An instance of this filter class
	 */
	public final static function createTestConfigurationLoadableClassesFilter () {
		// Get a new instance
		$filterInstance = new TestConfigurationLoadableClassesFilter();

		// Return the instance
		return $filterInstance;
	}

	/**
	 * Executes the filter with given request and response objects
	 *
	 * @param	$requestInstance	An instance of a class with an Requestable interface
	 * @param	$responseInstance	An instance of a class with an Responseable interface
	 * @return	void
	 * @todo	0% done
	 */
	public function execute (Requestable $requestInstance, Responseable $responseInstance) {
		// Init counter
		$passed = 0;
		$failed = 0;

		// Loop through all configuration keys
		foreach (FrameworkBootstrap::getConfigurationInstance()->getConfigurationArray() as $configKey => $configValue) {
			// Key must end with _class
			if (substr($configKey, -6, 6) != '_class') {
				// Skip this
				continue;
			} // END - if

			// Output message
			self::createDebugInstance(__CLASS__, __LINE__)->debugOutput(sprintf('Testing configKey=%s,configValue[%s]=%s', $configKey, gettype($configValue), $configValue));

			// This may throw exceptions
			try {
				// Is the config entry valid and class is there?
				if (!is_string($configKey)) {
					// Is not a string
					self::createDebugInstance(__CLASS__, __LINE__)->debugOutput(sprintf('configKey=%s has unexpected type "%s". FAILED', $configKey, gettype($configKey)));

					// Skip further tests
					$failed++;
					continue;
				} elseif (!class_exists($configValue)) {
					// Class not found
					self::createDebugInstance(__CLASS__, __LINE__)->debugOutput(sprintf('Class "%s" not found. FAILED', $configValue));

					// Skip further tests
					$failed++;
					continue;
				}
			} catch (InvalidArgumentException $e) {
				// Maybe not conform?
				self::createDebugInstance(__CLASS__, __LINE__)->debugOutput(sprintf('Class "%s" failed to load. Message: "%s"', $configValue, $e->getMessage()));

				// Skip further tests
				$failed++;
				continue;
			}

			// class_exists() didn't fail
			self::createDebugInstance(__CLASS__, __LINE__)->debugOutput(sprintf('Class "%s" loaded successfully. OKAY', $configValue));
			$passed++;
		} // END - foreach

		// Output result
		self::createDebugInstance(__CLASS__, __LINE__)->debugOutput(sprintf('Test result: %d okay, %d failed (%0.02f%% passed)', $passed, $failed, ($passed / ($passed + $failed) * 100)));

		// Implement this!
		$this->partialStub('Please implement this method.');
	}

}
